@@ -41,8 +41,8 @@ gdal.UseExceptions()            # Make GDAL throw exceptions on error
 osr.UseExceptions()             # And OSR as well.
 
 
-from .constants import (EPSG_WEB_MERCATOR, GDALTRANSLATE,
-                        GDALWARP, TILE_SIDE)
+from .constants import (EPSG_WEB_MERCATOR, ESRI_102100_PROJ, ESRI_102113_PROJ,
+                        GDALTRANSLATE, GDALWARP, TILE_SIDE)
 from .exceptions import (GdalError, CalledGdalError, UnalignedInputError,
                          UnknownResamplingMethodError)
 from .types import Extents, GdalFormat, XY
@@ -92,7 +92,8 @@ def preprocess(inputfile, outputfile, band=None, spatial_ref=None,
         )
 
     # Warp
-    if spatial_ref is not None and dataset.GetSpatialReference() != spatial_ref:
+    if spatial_ref is not None and \
+            dataset.GetSpatialReference() != spatial_ref:
         functions.append(
             ('Reprojecting to EPSG:{0}'.format(spatial_ref.GetEPSGCode()),
              partial(warp,
@@ -178,9 +179,6 @@ def warp(inputfile, spatial_ref=None, cmd=GDALWARP, resampling=None,
     ]
 
     # Warping to Mercator.
-    #
-    # Note that EPSG:3857 replaces this EPSG:3785 but GDAL doesn't know about
-    # it yet.
     if spatial_ref is None:
         spatial_ref = SpatialReference.FromEPSG(EPSG_WEB_MERCATOR)
     warp_cmd.extend(['-t_srs', spatial_ref.GetEPSGString()])
@@ -437,7 +435,19 @@ class Dataset(gdal.Dataset):
                     dataset=self)
 
     def GetSpatialReference(self):
-        return SpatialReference(self.GetProjection())
+        try:
+            sr = SpatialReference(self.GetProjection())
+            sr.AutoIdentifyEPSG()
+            return sr
+        except RuntimeError as re:
+            if 'Unsupported SRS' in re.message:
+                projcs_name = sr.GetAttrValue(str('PROJCS'))
+                # Returning equivalent EPSG code
+                if projcs_name == ESRI_102100_PROJ:
+                    return SpatialReference.FromEPSG(3857)
+                elif projcs_name == ESRI_102113_PROJ:
+                    return SpatialReference.FromEPSG(3785)
+            raise GdalError(re.message)
 
     def GetCoordinateTransformation(self, dst_ref):
         return CoordinateTransformation(src_ref=self.GetSpatialReference(),
@@ -676,16 +686,14 @@ class Dataset(gdal.Dataset):
 
         # Correct for origin, because you can't do modular arithmetic on
         # half-tiles.
-        extents = self.GetTiledExtents(transform=transform,
-                                       resolution=resolution)
         left, bottom = spatial_ref.OffsetPoint(*extents.lower_left)
         right, top = spatial_ref.OffsetPoint(*extents.upper_right)
 
         # Divide by number of tiles
-        return Extents(lower_left=XY(int(left / tile_width),
-                                     int(bottom / tile_height)),
-                       upper_right=XY(int(right / tile_width),
-                                      int(top / tile_height)))
+        return Extents(lower_left=XY(int(round(left / tile_width)),
+                                     int(round(bottom / tile_height))),
+                       upper_right=XY(int(round(right / tile_width)),
+                                      int(round(top / tile_height))))
 
     def GetWorldScalingRatios(self, resolution=None, places=None):
         """
@@ -784,6 +792,18 @@ class SpatialReference(osr.SpatialReference):
         epsg_string = self.GetEPSGString()
         if epsg_string:
             return int(epsg_string.split(':')[1])
+        else:
+            # HACK: The following is to cope with the fact that the Wkt
+            #       representation of the Web Mercator is different for
+            #       ESRI's WKID 102100 and EPSG's 3857 while both are
+            #       equivalent. Yet, Proj4 does not understand ESRI's
+            #       'mercator_auxiliary_sphere' projection name.
+            projcs_name = self.GetAttrValue(str('PROJCS'))
+            # Returning equivalent EPSG code
+            if projcs_name == ESRI_102100_PROJ:
+                return 3857
+            elif projcs_name == ESRI_102113_PROJ:
+                return 3785
 
     def GetEPSGString(self):
         if self.IsLocal() == 1:
@@ -793,8 +813,14 @@ class SpatialReference(osr.SpatialReference):
             cstype = b'GEOGCS'
         else:
             cstype = b'PROJCS'
-        return '{0}:{1}'.format(self.GetAuthorityName(cstype),
-                                self.GetAuthorityCode(cstype))
+
+        authority_name = self.GetAuthorityName(cstype)
+        authority_code = self.GetAuthorityCode(cstype)
+
+        if authority_name and authority_code:
+            return '{0}:{1}'.format(authority_name, authority_code)
+        else:
+            return None
 
     def GetMajorCircumference(self):
         if self.IsProjected() == 0:
@@ -804,7 +830,14 @@ class SpatialReference(osr.SpatialReference):
     def GetMinorCircumference(self):
         if self.IsProjected() == 0:
             return 2 * pi / self.GetAngularUnits()
-        return self.GetSemiMinor() * 2 * pi / self.GetLinearUnits()
+
+        semi_minor = self.GetSemiMinor() * 2 * pi / self.GetLinearUnits()
+        if self.GetEPSGCode() == 3857:
+            # Cancel the flattening of the spheroid.
+            # This is to account for the web mercator projection of points on
+            # a spheroid but interpretation of said points on a sphere.
+            return semi_minor / (1 - 1 / self.GetInvFlattening())
+        return semi_minor
 
     def GetWorldExtents(self):
         major = self.GetMajorCircumference() / 2
@@ -882,7 +915,7 @@ class VRT(object):
                     cmd,
                     '-q',                   # Quiet - FIXME: Use logging
                     '-of', 'GTiff',         # Output to GeoTIFF
-                    '-co', 'BIGTIFF=IF_NEEDED',  # Use BigTIFF if needed
+                    '-co', 'BIGTIFF=IF_SAFER',  # Use BigTIFF if >2GB
                     # gdal_translate does not support the following
                     # '-multi',               # Use multiple processes
                     # '-overwrite',           # Overwrite outputfile
